/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"

#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

static int isGroupMember(struct passwd* user, gid_t check) {
    static gid_t* groups = NULL;
    static int num = 0;

    if(user->pw_gid == check)
        return 1;

    if(!groups) {
        num = getgroups(0, NULL);
        groups = malloc(sizeof(gid_t) * num);

        if(!groups) {
            E("malloc error: %d (%s)\n", errno, strerror(errno));
            return 0;
        }

        if(getgroups(num, groups) == -1) {
            E("getgroups error: %d (%s)\n", errno, strerror(errno));
            free(groups);
            groups = NULL;
            return 0;
        }
    }

    {
        int i;
        for(i = 0; i < num; ++i) {
            if(groups[i] == user->pw_gid)
                return 1;
        }
    }

    return 0;
}

ITX_DEF_AND_OVERLOAD(int, access, (const char* path, int mode)) {
    int res = -1;
    struct stat st;
    struct passwd* pwd;

	DT("enter: path: %s, mode: %d\n", path, mode);

    if(stat(path, &st) != 0)
        goto done;

    pwd = getpwuid(geteuid());

    if(!pwd)
        goto done;

    if(pwd->pw_uid == 0 && (mode & X_OK) == 0) {
        // root may read/write any file.
        DT("root may read/write mode %x\n", mode);
        res = 0;
        goto done;
    }

    if(pwd->pw_uid == 0 && st.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
        // root can execute all that have any x bit set.
        DT("root may execute mode %x\n", mode);
        res = 0;
        goto done;
    }

    if(st.st_uid == pwd->pw_uid) {
        // the user is the owner
        DT("checking owner mode on %s\n", path);
        mode <<= 6;
    } else if(isGroupMember(pwd, st.st_gid)) {
        // user is in one of the groups.
        DT("checking group mode on %s\n", path);
        mode <<= 3;
    } else {
        DT("checking other mode on %s\n", path);
    }

    if(mode == 0 || st.st_mode & mode) {
        // either user or group or other bits matched.
        res = 0;
        goto done;
    }

    // no rights.
    errno = EACCES;

  done:
    DT("leave: %d\n", res);
    return res;
}
