/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"
#include "actcache.h"

#include <string.h> // for memset (used by FD_ZERO)
#include <sys/select.h>

static fd_set g_read_cache;
static fd_set g_write_cache;
static int g_inited = 0;

static fd_set* _GetCache(cachetype_t type) {
	if(g_inited == 0) {
		FD_ZERO(&g_read_cache);
		FD_ZERO(&g_write_cache);

		g_inited = 1;
	}

	switch(type) {
	case ActCache_Read:
		return &g_read_cache;
	case ActCache_Write:
		return &g_write_cache;
	}

	E("unknown or corrupted cache type (%d)!\n", type);
}

void ActCacheSet(cachetype_t type, int fd) {
	if(fd < 0 || fd >= FD_SETSIZE) {
		DC("invalid fd %d!\n", fd);
		return;
	}

	DC("adding fd %d to activity cache\n", fd);
	FD_SET(fd, _GetCache(type));
}

void ActCacheClear(cachetype_t type, int fd) {
	if(fd < 0 || fd >= FD_SETSIZE) {
		DC("invalid fd %d!\n", fd);
		return;
	}

	DC("clearing fd %d from activity cache\n", fd);
	FD_CLR(fd, _GetCache(type));
}

int ActCacheIsSet(cachetype_t type, int fd) {

	// ATTENTION: avoid crashing or misbehaving here. programs could
	//            pass invalid fd's easily and check for EBADF after
	//            the facts.
	if(fd < 0 || fd >= FD_SETSIZE) {
		DC("invalid fd %d!\n", fd);
		return 0;
	}

	DC("fd %d is %s in %s cache.\n", fd, FD_ISSET(fd, _GetCache(type)) ? "set" : "not set", (type == ActCache_Read) ? "read" : "write");
	return FD_ISSET(fd, _GetCache(type));
}
