/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"
#include <sys/stat.h>
#include <unistd.h>
#include <limits.h>
#include <string.h>
#include <fcntl.h>
#include <poll.h>
#include <sys/procfs.h>
#include <errno.h>

//
// the following are some helper functions for the debug output, etc.

#ifdef _DEBUG
# define TOKSTR " \t|,"

/* careful - hack */
extern char ** __argv;
static char* myname = NULL;

/* careful - another hack */
int _dbg_init_lock=0;

static void append_suffix(char* fn, char* suffix_spec) {
    char * token = strtok(suffix_spec, TOKSTR);
    while(token != NULL) {
        if(strcmp(token, "pid") == 0) {
            char pid_str[32];
            strcat(fn, ".");
            snprintf(pid_str, 31, "%d", getpid());
            strcat(fn, pid_str);
        } else if (strcmp(token, "name") == 0) {
            char* slash = strrchr(myname, '/');

            if(slash) {
                slash++;
            } else {
                slash = myname;
            }

            strcat(fn, ".");
            strcat(fn, slash);
        }

        token = strtok(NULL, TOKSTR);
    }
}

static int should_filter(char* ev, char* name) {
    char * token = NULL;

    /* never filter if no filter set. */
    if(!ev)
        return 0;

    token = strtok(ev, TOKSTR);

    while(token) {
        /* found in allowed names, don't filter */
        if(strstr(name, token) != NULL)
            return 0;

        token = strtok(NULL, TOKSTR);
    }

    /* not found - should filter out! */
    return 1;
}

FILE* DbgGetStream() {
    static FILE* stream = NULL;
	static int _init_level = 0;

    if(!stream) {
        int is_null_stream = 0;
        char* ev = getenv("SUACOMP_DEBUG_OUT");

		if(_init_level > 0) {
			// emergency save guard!
			return stderr;
		}

		_init_level++;

        if(!myname) {
            myname = __argv[0];
        }

        if(!ev)
            ev = "null";

        char* filter_ev = getenv("SUACOMP_DEBUG_FILTER");
        if(should_filter(filter_ev, myname)) {
            // disable output for this process.
            ev = "null";
        }

        if(strcmp(ev, "stdout") == 0) {
            stream = stdout;
        } else if(strcmp(ev, "stderr") == 0) {
            stream = stderr;
        } else if(strcmp(ev, "null") == 0 || strcmp(ev, "/dev/null") == 0) {
            stream = fopen("/dev/null", "w");
            is_null_stream = 1;
        } else {
            char fn[PATH_MAX];
            char* mode = getenv("SUACOMP_DEBUG_OUT_MODE");
            char* suffix = getenv("SUACOMP_DEBUG_OUT_SUFFIX");
            if(!mode) mode = "w";

            strcpy(fn, ev);

            append_suffix(fn, suffix);

            stream = fopen(fn, mode);

            if(!stream) {
                fprintf(stderr, "error opening debug stream at %s: %d (%s)\n", fn, errno, strerror(errno));
                stream = fopen("/dev/null", "w");
            }
        }

		_init_level--;

		if(!stream) {
			stream = stderr;
			D("warning: could not open debug stream!\n", NULL);
		}

        /* check buffering */
        {
            char* buffering = getenv("SUACOMP_DEBUG_UNBUFFERED");
            if(buffering != NULL && (
                    strcmp(buffering, "y") == 0 ||
                    strcmp(buffering, "yes") == 0 ||
                    strcmp(buffering, "true") == 0 ||
                    strcmp(buffering, "1") == 0 ||
                    strcmp(buffering, "on") == 0)) {
                setvbuf(stream, NULL, _IONBF, 0);
            }
        }

        /* careful - recursion ;) */
        D("init debug stream %s for process %d (%s)\n", (ev == NULL ? "stderr" : ev), getpid(), myname);
    }

    return stream;
}
#endif

// This wraps a whole lot of symbols from the libc, for debugging
// purposes only, this it does not influence the behaviour in any
// way, but prints nice messages when built with _DEBUG (and doesn't
// even overload the symbol in the !_DEBUG builds!).
//
// PLEASE KEEP THIS IN ALPHABETICAL ORDER!

DBGWRAPV (                      abort,      (void), (), "", NULL);
DBGWRAP  (unsigned int, "%d",   alarm,      (unsigned int x), (x), "%d", x);

// TODO: variadic functions! will probably end up creating a naked function manually pushing all
//       registers, and restoring after a call to DT()
// DBGWRAP(int, "%d", open,  (const char* path, int o_flag, ...), (path, o_flag, ...), "%s, %d, ...", path, o_flag);
