/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"
#include "actcache.h"
#include "sync.h"
#include "futimes.h"

#include <errno.h>
#include <stdio.h>

ITX_DEF_AND_OVERLOAD(int, fputs, (const char* buf, FILE* fp)) {
    int ret;
	ITX_REAL_FUNC(fputs);

	DT("enter: buf: %p (%s), fp: %p\n", buf, buf ? buf : "NULL", fp);

    sync_mark(fileno(fp));
    futimes_dirty(fileno(fp));

	if(ActCacheIsSet(ActCache_Write, fileno(fp))) {
		ITX_REPEAT(
			errno = 0;
			ret = real_fputs(buf, fp);
			D("real_fputs(%p, %p) = %d\n", buf, fp, ret);

			if(ret == 0) {
				D("ret = %d, errno = %d\n", ret, errno);

				if(errno == EAGAIN) {
					clearerr(fp);
					ITX_DELAYED_CONTINUE;
				}
			}

			break;
		)

		ActCacheClear(ActCache_Write, fileno(fp));
	} else {
		ret = real_fputs(buf, fp);
	}
    DT("leave: %d\n", ret);
    return ret;
}
