/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"
#include "actcache.h"
#include "sync.h"
#include "futimes.h"

#include <errno.h>
#include <stdio.h>

ITX_DEF_AND_OVERLOAD(size_t, fwrite, (const void* buf, size_t sz, size_t cnt, FILE* fp)) {
	ITX_REAL_FUNC(fwrite);

	DT("enter: buf: %p, sz: %d, cnt: %d, fp: %p\n", buf, sz, cnt, fp);

    sync_mark(fileno(fp));
    futimes_dirty(fileno(fp));

	if(ActCacheIsSet(ActCache_Write, fileno(fp))) {
		size_t ret = -1;

		ITX_REPEAT(
			errno = 0;
			ret = real_fwrite(buf, sz, cnt, fp);
			D("real_fwrite(%p, %d, %d, %p) = %d\n", buf, sz, cnt, fp, ret);

			if(ret == 0) {
				D("ret = %d, errno = %d\n", ret, errno);

				if(errno == EAGAIN) {
					clearerr(fp);
					ITX_DELAYED_CONTINUE;
				}
			}

			break;
		)

		ActCacheClear(ActCache_Write, fileno(fp));
        DT("leave: %d\n", ret);
		return ret;
	} else {
        size_t ret;
		ret = real_fwrite(buf, sz, cnt, fp);
        DT("leave: %d\n", ret);
		return ret;
	}
}
