/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#ifndef __SC_STDINT_H__
#define __SC_STDINT_H__

/* on interix, sys/types.h unconditionally typedefs most
 * of what we need. to avoid conflicts, include sys/types.h
 * here. */
#include "unistd.h"
#include <sys/types.h>

/* only 32 bit interix is supported by this file ! */
#define __STDINT_BITS   32

#define ____STDINT_BITTYPE(type, bits, name)    typedef type ## bits ## _t name;
#define ___STDINT_BITTYPE(type, bits, name)     ____STDINT_BITTYPE(type, bits, name)
#define __STDINT_S_BITTYPE(name)                ___STDINT_BITTYPE(int, __STDINT_BITS, name)
#define __STDINT_U_BITTYPE(name)                ___STDINT_BITTYPE(uint, __STDINT_BITS, name)

#define ____STDINT_BITLIMIT(type, bits, mm)       type ## bits ## mm
#define ___STDINT_BITLIMIT(type, bits, mm)      ____STDINT_BITLIMIT(type, bits, mm)
#define __STDINT_S_BITLIMIT(x)                  ___STDINT_BITLIMIT(INT, __STDINT_BITS, _ ## x)
#define __STDINT_U_BITLIMIT(x)                  ___STDINT_BITLIMIT(UINT, __STDINT_BITS, _ ## x)

#define __STDINT_DERIVED_TYPES(x) \
    typedef int ## x ## _t                  int_least ## x ## _t; \
    typedef uint ## x ## _t                uint_least ## x ## _t; \
    typedef int ## x ## _t                  int_fast ## x ## _t;  \
    typedef uint ## x ## _t                uint_fast ## x ## _t;  \

#ifdef __GNUC__
/* attention: sys/types.h misses this: */
typedef unsigned long long      uint64_t;
#endif

__STDINT_DERIVED_TYPES(8)
__STDINT_DERIVED_TYPES(16)
__STDINT_DERIVED_TYPES(32)

#ifndef _SIZE_T_DEFINED
__STDINT_U_BITTYPE(size_t);
#define _SIZE_T_DEFINED
#endif /* _SIZE_T_DEFINED */

#ifndef _SSIZE_T_DEFINED
__STDINT_S_BITTYPE(ssize_t);
#define _SSIZE_T_DEFINED
#endif /* _SSIZE_T_DEFINED */

#ifndef _INTPTR_T_DEFINED
__STDINT_S_BITTYPE(intptr_t);
#define _INTPTR_T_DEFINED
#endif /* _INTPTR_T_DEFINED */

#ifndef _UINTPTR_T_DEFINED
__STDINT_U_BITTYPE(uintptr_t);
#define _UINTPTR_T_DEFINED
#endif /* _UINTPTR_T_DEFINED */

#ifndef _PTRDIFF_T_DEFINED
#ifndef _PTRDIFF_T /* < stddef.h */
__STDINT_S_BITTYPE(ptrdiff_t);
#define _PTRDIFF_T
#endif /* _PTRDIFF_T */
#define _PTRDIFF_T_DEFINED
#endif /* _PTRDIFF_T_DEFINED */

#ifdef __GNUC__
__STDINT_DERIVED_TYPES(64)
typedef int64_t                 intmax_t;
typedef uint64_t               uintmax_t;
#else
typedef int32_t                 intmax_t;
typedef uint32_t               uintmax_t;
#endif

#if !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS)

#define __INT8_C(v)	        v
#define __INT16_C(v)	    v
#define __INT32_C(v)	    v
#define __INT64_C(v)	    v ## LL

#define __UINT8_C(v)	    v ## U
#define __UINT16_C(v)	    v ## U
#define __UINT32_C(v)	    v ## U
#define __UINT64_C(v)	    v ## ULL

#define INT8_MAX	        __INT8_C(0x7f)
#define INT16_MAX	        __INT16_C(0x7fff)
#define INT32_MAX	        __INT32_C(0x7fffffff)
#define INT64_MAX	        __INT64_C(0x7fffffffffffffff)

#define INT8_MIN	        __INT8_C(-0x80)
#define INT16_MIN	        __INT16_C(-0x8000)
#define INT32_MIN	        __INT32_C(-0x80000000)
#define INT64_MIN	        __INT64_C(-0x8000000000000000)

#define UINT8_MAX	        __UINT8_C(0xff)
#define UINT16_MAX	        __UINT16_C(0xffff)
#define UINT32_MAX	        __UINT32_C(0xffffffff)
#define UINT64_MAX	        __UINT64_C(0xffffffffffffffff)

#define INT_LEAST8_MIN	    INT8_MIN
#define INT_LEAST16_MIN	    INT16_MIN
#define INT_LEAST32_MIN	    INT32_MIN
#define INT_LEAST64_MIN	    INT64_MIN

#define INT_LEAST8_MAX	    INT8_MAX
#define INT_LEAST16_MAX	    INT16_MAX
#define INT_LEAST32_MAX	    INT32_MAX
#define INT_LEAST64_MAX	    INT64_MAX

#define UINT_LEAST8_MAX		UINT8_MAX
#define UINT_LEAST16_MAX	UINT16_MAX
#define UINT_LEAST32_MAX	UINT32_MAX
#define UINT_LEAST64_MAX	UINT64_MAX

#define INT_FAST8_MIN	    INT32_MIN
#define INT_FAST16_MIN	    INT32_MIN
#define INT_FAST32_MIN	    INT32_MIN
#define INT_FAST64_MIN	    INT64_MIN

#define INT_FAST8_MAX	    INT32_MAX
#define INT_FAST16_MAX	    INT32_MAX
#define INT_FAST32_MAX	    INT32_MAX
#define INT_FAST64_MAX	    INT64_MAX

#define UINT_FAST8_MAX	    UINT32_MAX
#define UINT_FAST16_MAX	    UINT32_MAX
#define UINT_FAST32_MAX	    UINT32_MAX
#define UINT_FAST64_MAX	    UINT64_MAX

#define INTPTR_MIN	        __STDINT_S_BITLIMIT(MIN)
#define INTPTR_MAX	        __STDINT_S_BITLIMIT(MAX)
#define UINTPTR_MAX	        __STDINT_U_BITLIMIT(MAX)

#define PTRDIFF_MIN	        __STDINT_S_BITLIMIT(MIN)
#define PTRDIFF_MAX	        __STDINT_S_BITLIMIT(MAX)

#define SIZE_MAX	        __STDINT_U_BITLIMIT(MAX)

#define INTMAX_MIN	        INT64_MIN
#define INTMAX_MAX	        INT64_MAX
#define UINTMAX_MAX	        UINT64_MAX

#define SIG_ATOMIC_MIN	    INT32_MIN
#define SIG_ATOMIC_MAX	    INT32_MAX

#define WINT_MIN	        INT32_MIN
#define WINT_MAX	        INT32_MAX

#endif /* !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS) */

#if !defined(__cplusplus) || defined(__STDC_CONSTANT_MACROS)

#define INT8_C(v)	        v
#define INT16_C(v)	        v
#define INT32_C(v)	        v
#define INT64_C(v)	        v ## LL

#define UINT8_C(v)	        v ## U
#define UINT16_C(v)	        v ## U
#define UINT32_C(v)	        v ## U
#define UINT64_C(v)	        v ## ULL

#define INTMAX_C(v)	        v ## LL
#define UINTMAX_C(v)	    v ## ULL

#endif /* !defined(__cplusplus) || defined(__STDC_CONSTANT_MACROS) */

#endif

