/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"

#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include <pwd.h>

ITX_DEF(struct passwd*, getpwuid_ex, (uid_t uid, unsigned long flags));

char* GetLocalAdminName(int full) {
	static char* name = NULL;
	static char* full_name = NULL;
	ITX_REAL_FUNC(getpwuid_ex);

	if(!name || !full_name) {
		struct passwd* tmp;
		if((tmp = real_getpwuid_ex(ITX_ROOT_UID, 0)) == NULL)
			E("cannot get real local administrator name (uid: %d, errno: %d)!\n", ITX_ROOT_UID, errno);

		name = strdup(tmp->pw_name);

		if((tmp = real_getpwuid_ex(ITX_ROOT_UID, PW_FULLNAME)) == NULL)
			E("cannot get real local administrator name (uid: %d, errno: %d)!\n", ITX_ROOT_UID, errno);

		full_name = strdup(tmp->pw_name);
	}

	return full ? full_name : name;
}

void ApplyPasswd(struct passwd* in, struct passwd* out, int shallow) {
	#define APPLY(x, y, a) if(x->a && y->a) { if(shallow) { y->a = x->a; } else { strcpy(y->a, x->a); } } else { y->a = NULL; }

	APPLY(in, out, pw_name);
	APPLY(in, out, pw_dir);
	APPLY(in, out, pw_shell);
	APPLY(in, out, pw_passwd);
	APPLY(in, out, pw_gecos);

	#undef APPLY
	#define APPLY(x, y, a) y->a = x->a;

	APPLY(in, out, pw_uid);
	APPLY(in, out, pw_gid);
	APPLY(in, out, pw_change);
	APPLY(in, out, pw_expire);

	#undef APPLY

	{ int i; for(i = 0; i < 20; ++i) { out->reserved[i] = in->reserved[i]; } }
}

static struct passwd* LocalCopyPasswd(struct passwd* in) {
	static char pw_name[LOGIN_NAME_MAX];
	static char pw_dir[PATH_MAX];
	static char pw_shell[PATH_MAX];
	static char pw_passwd[2] = "*"; // according to man page...
	static char pw_gecos[4096]; // guess a large value... this is the user description in the windows udb.
	static struct passwd gpw;

	if(!in) return in;

	// explicitly re-set, since we pass out the address of the struct,
	// so it might have been modified!
	gpw.pw_name   = pw_name;
	gpw.pw_dir    = pw_dir;
	gpw.pw_shell  = pw_shell;
	gpw.pw_passwd = pw_passwd;
	gpw.pw_gecos  = pw_gecos;

	ApplyPasswd(in, &gpw, 0);
	
	if(strncmp(in->pw_passwd, gpw.pw_passwd, sizeof(pw_passwd)) != 0) {
		E("pw_passwd is not '*'! it is '%s', which is not valid according to the man page!\n", in->pw_passwd);
	}

	return &gpw;
}

void MapPasswd(struct passwd* pwd, int to_win) {
	struct passwd* mypw;

	if(!pwd)
		return;

	DM("passwd in: to_win: %d, pwd: { %s, %s, %d, %d }\n", 
        to_win, pwd->pw_name, pwd->pw_dir, pwd->pw_uid, pwd->pw_gid);

	// ATTENTION: pwd->pw_name may be a pointer to a static buffer, so let's keep our own
	//            buffer, since we _may_ need to call getpwuid_ex to determine the local
	//            administrators name.
	mypw = LocalCopyPasswd(pwd);

	mypw->pw_name = (char*)MapUserName(mypw->pw_name, to_win);

	ITX_MAP_UID(mypw->pw_uid, to_win);
	ITX_MAP_GID(mypw->pw_gid, to_win);

	// may point to another passwd than the input parameter!
	ApplyPasswd(mypw, pwd, 1);

	DM("passwd out: to_win: %d, pwd: { %s -> %s, %s -> %s, %d -> %d, %d -> %d }\n", 
        to_win, pwd->pw_name, mypw->pw_name, pwd->pw_dir, mypw->pw_dir, pwd->pw_uid, mypw->pw_uid, pwd->pw_gid, mypw->pw_gid);
}

char const* MapUserName(char const* name, int to_win) {
	if(to_win && (strcmp(name, "root") == 0)) {
		DM("mapping %s -> %s\n", "root", GetLocalAdminName(False));
		return GetLocalAdminName(False);
	} else if(!to_win && (strcmp(name, GetLocalAdminName(True)) == 0 || strcmp(name, GetLocalAdminName(False)) == 0)) {
		DM("mapping %s <- %s\n", "root", name);
		return "root";
	} else {
		DM("not mapping %s, not root\n", name);
		return name;
	}
}

