/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"
#include "actcache.h"
#include "sync.h"
#include "futimes.h"

#include <errno.h>
#include <unistd.h>

ITX_DEF_AND_OVERLOAD(ssize_t, pwrite, (int fd, const void* buf, size_t cnt, off_t off)) {
    size_t ret;
	ITX_REAL_FUNC(pwrite);

	DT("enter: fd: %d, buf: %p, cnt: %d, off: %ld\n", fd, buf, cnt, off);

    sync_mark(fd);
    futimes_dirty(fd);

	// If the fd is set in the cache, do the EAGAIN loop.
	if(ActCacheIsSet(ActCache_Write, fd)) {
		ITX_REPEAT(
			errno = 0;
			ret = real_pwrite(fd, buf, cnt, off);
			D("real_pwrite(%d, %p, %d, %ld) = %d\n", fd, buf, cnt, off, ret);

			if(ret == 0) {
				D("ret = %d, errno = %d\n", ret, errno);

				if(errno == EAGAIN)
					ITX_DELAYED_CONTINUE;
			}

			break;
		)
		
		ActCacheClear(ActCache_Write, fd);
	} else {
		ret = real_pwrite(fd, buf, cnt, off);
	}
    DT("leave: %d\n", ret);
    return ret;
}

