/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"
#include "actcache.h"

#include <errno.h>
#include <unistd.h>
#include <sys/uio.h>

ITX_DEF_AND_OVERLOAD(ssize_t, readv, (int fd, const struct iovec* iov, int iovcnt)) {
    size_t ret;
	ITX_REAL_FUNC(readv);

	DT("enter: fd: %d, iov: %p, iovcnt: %d\n", fd, iov, iovcnt);
	// If the fd is set in the cache, do the EAGAIN loop.
	if(ActCacheIsSet(ActCache_Read, fd)) {
		ITX_REPEAT(
			errno = 0;
			ret = real_readv(fd, iov, iovcnt);
			D("real_readv(%d, %p, %d) = %d\n", fd, iov, iovcnt, ret);

			if(ret == 0) {
				D("ret = %d, errno = %d\n", ret, errno);

				if(errno == EAGAIN)
					ITX_DELAYED_CONTINUE;
			}

			break;
		)
		
		ActCacheClear(ActCache_Read, fd);
	} else {
		ret = real_readv(fd, iov, iovcnt);
	}
    DT("leave: %d\n", ret);
    return ret;
}

