/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"
#include "actcache.h"

#include <sys/select.h>

ITX_DEF_AND_OVERLOAD(int, select, 
		(int nfds, fd_set* rfds, fd_set* wfds, fd_set* exfds, struct timeval* to)) {

	int ret = -1;
	ITX_REAL_FUNC(select);

	DT("enter: nfds: %d, rfds: %p, wfds: %p, exfds: %p, timeout: %p\n", nfds, rfds, wfds, exfds, to);

	// Another thing we can handle here: on interix 3.5 select with nfds = 0 is broken.
	// This is used to "sleep", but returns immediately on such interixen.
	if(nfds == 0) {
		nfds = 1;
	}

	// TODO: for now, exfds are ignored (wrt ActCache). implement if required.

	ITX_FOREACH_SET_FD(rfds, nfds, ActCacheClear(ActCache_Read, i););
	ITX_FOREACH_SET_FD(wfds, nfds, ActCacheClear(ActCache_Write, i););

	ret = real_select(nfds, rfds, wfds, exfds, to);
	if(to) {
		D("real_select(%d, %p, %p, %p, {%d:%d}) = %d\n", nfds, rfds, wfds, exfds, to->tv_sec, to->tv_usec, ret);
	} else {
		D("real_select(%d, %p, %p, %p, {null}) = %d\n", nfds, rfds, wfds, exfds, ret);
	}

	if(ret > 0) {
		// TODO: is isatty the right check, or do we need to check all non-blocking streams?
		// theoretically, we could mark _all_ streams, but to keep the smallest possible
		// overhead, avoid checked calls where possible
		ITX_FOREACH_SET_FD(rfds, nfds, if(isatty(i)) { ActCacheSet(ActCache_Read, i); } else { DC("%d is not a tty - not caching.", i); } );
		ITX_FOREACH_SET_FD(wfds, nfds, if(isatty(i)) { ActCacheSet(ActCache_Write, i); } else { DC("%d is not a tty - not caching.", i); } );
	}

    DT("leave: %d\n", ret);
	return ret;
}
