/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"

#include <unistd.h>
#include <sys/sem.h>
#include <stdarg.h>

ITX_DEF_AND_OVERLOAD(int, semctl, (int semid, int semnum, int cmd, ...)) {
    int res;
	ITX_REAL_FUNC(semctl);

	DT("enter: semid: %d, semnum: %d, cmd: %d, ...\n", semid, semnum, cmd);

	switch(cmd) {
	case SETVAL:  /* commands which require an int */
		{
			va_list args;
			va_start(args, cmd);

			res = real_semctl(semid, semnum, cmd, va_arg(args, int));

			va_end(args);
            break;
		}
	case GETALL:  /* commands which require an array */
	case SETALL:
		{
			va_list args;
			va_start(args, cmd);

			res = real_semctl(semid, semnum, cmd, va_arg(args, unsigned short*));

			va_end(args);
            break;
		}
	case IPC_STAT:
		{
			va_list args;
			struct semid_ds* ids;
			va_start(args, cmd);

			ids = va_arg(args, struct semid_ds*);
			res = real_semctl(semid, semnum, cmd, ids);

			ITX_MAP_UID(ids->sem_perm.uid, False);
			ITX_MAP_UID(ids->sem_perm.cuid, False);
			ITX_MAP_GID(ids->sem_perm.gid, False);
			ITX_MAP_GID(ids->sem_perm.cgid, False);

			va_end(args);
            break;
		}
	case IPC_SET:
		{
			va_list args;
			struct semid_ds* ids;
			va_start(args, cmd);

			ids = va_arg(args, struct semid_ds*);

			ITX_MAP_UID(ids->sem_perm.uid, True);
			ITX_MAP_UID(ids->sem_perm.cuid, True);
			ITX_MAP_GID(ids->sem_perm.gid, True);
			ITX_MAP_GID(ids->sem_perm.cgid, True);

			res = real_semctl(semid, semnum, cmd, ids);

			va_end(args);
            break;
		}
	default:	 /* commands which don't require any argument */
		res = real_semctl(semid, semnum, cmd);
        break;
	}
    DT("leave: %d\n", res);
    return res;
}
