/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#ifndef __SUACOMP_H__
#define __SUACOMP_H__

#ifndef _ALL_SOURCE
#define _ALL_SOURCE
#endif

#include <unistd.h> /* getpid() */
#include <stdlib.h> /* abort() */
#include <stdio.h> /* fprintf() */
#include <pwd.h>   /* struct passwd */
#include <grp.h>   /* struct group */

#include "include/suacomp.h"

#ifdef _DEBUG
# include <time.h>
extern FILE* DbgGetStream();
# define D(fmt, ...) { fprintf(DbgGetStream(), "%d: %d *** %15s:%d :: " fmt, getpid(), time(NULL), __FUNCTION__, __LINE__, __VA_ARGS__); }
#else
# define D(fmt, ...) {}
#endif

#define E(fmt, ...) { fprintf(stderr, "fatal: " fmt, __VA_ARGS__); abort(); }

#ifdef _DEBUG_MAPPING
# define DM(fmt, ...) D(fmt, __VA_ARGS__)
#else
# define DM(fmt, ...)
#endif

#ifdef _DEBUG_ACTCACHE
# define DC(fmt, ...) D(fmt, __VA_ARGS__)
#else
# define DC(fmt, ...)
#endif

#ifdef _DEBUG_DYNLOAD
# define DD(fmt, ...) D(fmt, __VA_ARGS__)
#else
# define DD(fmt, ...)
#endif

#ifdef _DEBUG_TRACE
# define DT(fmt, ...) D(fmt, __VA_ARGS__)
// The ... parameter (below) is for the debug dump arguments, should be printf syntax!
# define DBGWRAP(ret, ret_fmt, func, arg_typed_names, arg_names, ...) \
	ITX_DEF_AND_OVERLOAD(ret, func, arg_typed_names) { \
		ITX_REAL_FUNC(func); \
		DT("enter: " __VA_ARGS__); fprintf(DbgGetStream(), "\n"); \
		{	ret tmp = (real_##func)arg_names; \
			DT("leave: " ret_fmt "\n", tmp); \
			return tmp; \
		} \
	}
# define DBGWRAPV(func, arg_typed_names, arg_names, ...) \
    ITX_DEF_AND_OVERLOAD(void, func, arg_typed_names) { \
        ITX_REAL_FUNC(func); \
        DT("enter: " __VA_ARGS__); fprintf(DbgGetStream(), "\n"); \
        (real_##func)arg_names; \
        DT("leave: (void)\n", NULL); \
    }
#else
# define DT(fmt, ...)
# define DBGWRAP(ret, ret_fmt, func, arg_typed_names, args_names, ...)
# define DBGWRAPV(fuc, arg_typed_names, arg_names, ...)
#endif

#define ITX_MAP_UID(uid, to_win) \
	if(to_win && uid == 0) { DM("mapping uid %d -> %d\n", uid, ITX_ROOT_UID); uid = ITX_ROOT_UID; } else if(!to_win && uid == ITX_ROOT_UID) { DM("mapping uid %d <- %d\n", 0, uid); uid = 0; } else { DM("not mapping uid %d, not root\n", uid); }


// Maximum number of retries for each EAGAIN wrapped function.
#ifndef ITX_REPEAT_MAX
#define ITX_REPEAT_MAX (1024 * 1024)
#endif

// Enabled / Disable preloadable library support.
#ifndef ITX_PRELOADABLE
#define ITX_PRELOADABLE 0
#endif

// Delay between each repeat, to not stress the cpu too much.
#define ITX_DELAY { usleep(5); }
#define ITX_DELAYED_CONTINUE { ITX_DELAY; continue; }

// Repeat until max repeat is reached.
#define ITX_REPEAT(x) { int i; for(i = 0; i < ITX_REPEAT_MAX; ++i) { x } }

extern void* GetSysFunction(char const* name);

#ifdef ITX_STATIC
# define ITX_FUNCTION(x) &(_##x)
# define ITX_DEF(ret, name, args) \
	extern ret (_##name) args;       \
	typedef ret (*name##Func)args;
# define ITX_OVERLOAD(ret, name, args) ret (name) args
#else
# define ITX_FUNCTION(x) GetSysFunction(#x)
# define ITX_DEF(ret, name, args) \
	typedef ret (*name##Func)args;
# define ITX_OVERLOAD(ret, name, args) ret (_##name) args
#endif

#define ITX_DEF_AND_OVERLOAD(ret, name, args) \
	ITX_DEF(ret, name, args) \
	ITX_OVERLOAD(ret, name, args)

#define ITX_REAL_FUNC(name) \
	static name##Func real_##name = NULL; \
	if(real_##name == NULL) { real_##name = (name##Func)ITX_FUNCTION(name); }

#endif

// UID / GID mapping

#define ITX_ROOT_UID 197108
#define ITX_ROOT_GID 131616

// helper :)
#ifndef True
#define True 1
#endif
#ifndef False
#define False 0
#endif

#define ITX_MAP_GID(gid, to_win) \
	if(to_win && gid == 0) { DM("mapping gid %d -> %d\n", gid, ITX_ROOT_GID); gid = ITX_ROOT_GID; } else if(!to_win && gid == ITX_ROOT_GID) { DM("mapping gid %d <- %d\n", 0, gid); gid = 0; } else { DM("not mapping gid %d, not root\n", gid); }

#define ITX_MAP_IDS(uid, gid, to_win) \
	ITX_MAP_UID(uid, to_win); \
	ITX_MAP_GID(gid, to_win);

#define ITX_MAP_RETURNOF(func, args, is_uid, to_win) \
	if(is_uid) { \
		uid_t res = (func)args; \
		ITX_MAP_UID(res, to_win); \
        DT("leave: %d\n", res); \
		return res; \
	} else { \
		gid_t res = (func)args; \
		ITX_MAP_GID(res, to_win); \
        DT("leave: %d\n", res); \
		return res; \
	}

#define ITX_MAP_RETURNOF_PASSWD(func, args, to_win) \
	{	struct passwd* pwd = (func)args; \
		MapPasswd(pwd, to_win); \
        if(pwd != NULL) { \
            DT("leave: { name: %s, uid: %d, gid: %d }\n", pwd->pw_name, pwd->pw_uid, pwd->pw_gid); \
        } else  { \
            DT("leave: NULL\n", NULL); } \
		return pwd; }

#define ITX_MAP_RETURNOF_GROUP(func, args, to_win) \
	{	struct group* grp = (func)args; \
		MapGroup(grp, to_win); \
        if(grp != NULL) { \
            DT("leave: { name: %s, gid: %d }\n", grp->gr_name, grp->gr_gid); \
        } else { \
            DT("leave: NULL\n", NULL); } \
		return grp; }

/* uses char* to be able to service both char* and const char* returning
 * functions. */
#define ITX_MAP_RETURNOF_NAME(func, args, to_win) \
    {   char* name = (func)args; \
        name = (char*)MapUserName(name, to_win); \
        DT("leave: %s\n", (name? name : "NULL")); \
        return name; }

extern char* GetLocalAdminName(int full);
extern char const* MapUserName(char const* name, int to_win);
extern void MapPasswd(struct passwd* pwd, int to_win);

extern char const* MapGroupName(char const* name, int to_win);
extern void MapGroup(struct group* grp, int to_win);

