/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"
#include "actcache.h"

#include <string.h>
#include <sys/queue.h>

static int g_inited = 0;
static fd_set g_sync_queue;

void sync(void) {
    ITX_FOREACH_SET_FD(&g_sync_queue, FD_SETSIZE, {
        DT("flushing %d\n", i);
        fsync(i);
        FD_CLR(i, &g_sync_queue);
    })
}

void sync_mark(int fd) {
	if(fd < 0 || fd >= FD_SETSIZE) {
		DC("invalid fd %d!\n", fd);
		return;
	}

    DT("marking %d for sync()\n", fd);

    if(g_inited == 0) {
        FD_ZERO(&g_sync_queue);
        g_inited = 1;
    }

    FD_SET(fd, &g_sync_queue);
}

void sync_unmark(int fd) {
	if(fd < 0 || fd >= FD_SETSIZE) {
		DC("invalid fd %d!\n", fd);
		return;
	}

    DT("unmarking %d for sync()\n", fd);

    if(!g_inited)
        return;

    FD_CLR(fd, &g_sync_queue);
}
