/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "test.h"
#include "../suacomp.h"
#include "../actcache.h"

#include <unistd.h>
#include <sys/stat.h>
#include <string.h>
#include <pty.h>
#include <fcntl.h>
#include <errno.h>

void test_actcache(int argc, char** argv) {
	ActCacheSet(ActCache_Read, 5);
	EXP_SUCCESS(ActCacheIsSet(ActCache_Read, 5) != 0);
	EXP_SUCCESS(ActCacheIsSet(ActCache_Read, -1) == 0);

	ActCacheClear(ActCache_Read, 5);
	EXP_SUCCESS(ActCacheIsSet(ActCache_Read, 5) == 0);

	// now for the real meat :)
	{
		// need a pty, as the actcache is only used when isatty(fd).
		int master, slave;
		char data[] = { "testdata" };
		char buf[sizeof(data)+1];
		fd_set rset, wset;
		struct timeval to = { 0,0 };
		struct timeval lto = { 10,0 };

		EXP_SUCCESS(openpty(&master, &slave, NULL, NULL, NULL) == 0);

		// set non-blocking to prevent the test from hanging.
		EXP_SUCCESS(fcntl(master, F_SETFL, fcntl(master, F_GETFL, 0) | O_NONBLOCK) != -1);
		EXP_SUCCESS(fcntl(slave, F_SETFL, fcntl(slave, F_GETFL, 0) | O_NONBLOCK) != -1);
		
		FD_ZERO(&rset);
		FD_ZERO(&wset);
		FD_SET(master, &rset);
		FD_SET(slave, &wset);

		EXP_SUCCESS(!ActCacheIsSet(ActCache_Read, master));
		EXP_SUCCESS(!ActCacheIsSet(ActCache_Write, slave));

		EXP_SUCCESS(select(slave+1, NULL, &wset, NULL, &to) == 1);
		EXP_SUCCESS(ActCacheIsSet(ActCache_Write, slave));

		EXP_SUCCESS(select(master+1, &rset, NULL, NULL, &to) == 0);

		EXP_SUCCESS(write(slave, data, sizeof(data)) == sizeof(data));
		EXP_SUCCESS(!ActCacheIsSet(ActCache_Write, slave));

		FD_ZERO(&rset);
		FD_ZERO(&wset);
		FD_SET(master, &rset);
		FD_SET(slave, &wset);

		EXP_SUCCESS(select(master+1, &rset, NULL, NULL, &lto) == 1);
		EXP_SUCCESS(ActCacheIsSet(ActCache_Read, master));

		{ 
			int nread = read(master, buf, sizeof(data)); 
			VMSG("read(%d, %p, %d) = %d, errno: %d\n", master, buf, sizeof(data), nread, errno);
			EXP_SUCCESS(nread == sizeof(data)); 
		}

		EXP_SUCCESS(!ActCacheIsSet(ActCache_Read, master));

		EXP_SUCCESS(select(slave+1, NULL, &wset, NULL, &to) == 1);
		EXP_SUCCESS(ActCacheIsSet(ActCache_Write, slave));
		
		close(master);
		close(slave);

		EXP_SUCCESS(ActCacheIsSet(ActCache_Write, slave) == 0);
	}
}
