/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "test.h"
#include "../suacomp.h"
#include "../include/sys/time.h"

#include <unistd.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <string.h>

#define F "test/.test.futimes"

void test_futimes(int argc, char** argv) {
    struct stat st;
    struct timeval tv[2];
    FILE* fp;
    time_t secs;

    unlink(F);
    fp = fopen(F, "wb+");

    EXP_SUCCESS(fp != NULL);
    EXP_SUCCESS(stat(F, &st) != -1);
    DT("file mtime: %d\n", st.st_mtime);

    tv[0].tv_sec = st.st_atime - 10;
    tv[0].tv_usec = 0;
    tv[1].tv_sec = st.st_mtime - 20;
    tv[1].tv_usec = 0;

    // test simple case.
    EXP_SUCCESS(futimes(fileno(fp), tv) != -1);
    EXP_SUCCESS(stat(F, &st) != -1);
    DT("file mtime: %d\n", st.st_mtime);

    EXP_SUCCESS(tv[0].tv_sec == st.st_atime);
    EXP_SUCCESS(tv[1].tv_sec == st.st_mtime);

    // test dirtying file timestamps.
    EXP_SUCCESS(fwrite("huhu", 5, 1, fp) != -1);
    fflush(fp);

    EXP_SUCCESS(stat(F, &st) != -1);
    EXP_SUCCESS(tv[1].tv_sec != st.st_mtime);

    // test good timestamp after sync w/o modification of file.
    EXP_SUCCESS(futimes(fileno(fp), tv) != -1);
    fsync(fileno(fp));

    EXP_SUCCESS(stat(F, &st) != -1);
    DT("file mtime: %d\n", st.st_mtime);

    EXP_SUCCESS(tv[0].tv_sec == st.st_atime);
    EXP_SUCCESS(tv[1].tv_sec == st.st_mtime);

    // test good timestamp after flush w/o modification of file.
    EXP_SUCCESS(futimes(fileno(fp), tv) != -1);
    fflush(fp); 

    EXP_SUCCESS(stat(F, &st) != -1);
    DT("file mtime: %d\n", st.st_mtime);

    EXP_SUCCESS(tv[0].tv_sec == st.st_atime);
    EXP_SUCCESS(tv[1].tv_sec == st.st_mtime);

    // test reset timestamp after flush w/ modification.
    EXP_SUCCESS(futimes(fileno(fp), tv) != -1);
    fwrite("xx", 3, 1, fp);
    fflush(fp);

    EXP_SUCCESS(stat(F, &st) != -1);
    DT("file mtime: %d\n", st.st_mtime);

    EXP_SUCCESS(tv[1].tv_sec != st.st_mtime);

    // test good filestamp after closing of file w/o modification.
    EXP_SUCCESS(futimes(fileno(fp), tv) != -1);
    fclose(fp);

    EXP_SUCCESS(stat(F, &st) != -1);
    DT("file mtime: %d\n", st.st_mtime);

    EXP_SUCCESS(tv[0].tv_sec == st.st_atime);
    EXP_SUCCESS(tv[1].tv_sec == st.st_mtime);

    unlink(F);
}

