/* ----------------------------------------------------------------------------
 *  "THE BEER-WARE LICENSE" (Revision 42):
 *  <mduft@gentoo.org> wrote this file.  As long as you retain this notice you
 *  can do whatever you want with this stuff. If we meet some day, and you think
 *  this stuff is worth it, you can buy me a beer in return.   Markus Duft
 *  ---------------------------------------------------------------------------- */

#include "suacomp.h"
#include "actcache.h"
#include "sync.h"
#include "futimes.h"

#include <errno.h>
#include <unistd.h>
#include <sys/uio.h>

ITX_DEF_AND_OVERLOAD(ssize_t, writev, (int fd, const struct iovec* iov, int iovcnt)) {
	ITX_REAL_FUNC(writev);

	DT("enter: fd: %d, iov: %p, iovcnt: %d\n", fd, iov, iovcnt);

    sync_mark(fd);
    futimes_dirty(fd);

	// If the fd is set in the cache, do the EAGAIN loop.
	if(ActCacheIsSet(ActCache_Write, fd)) {
		size_t ret = -1;

		ITX_REPEAT(
			errno = 0;
			ret = real_writev(fd, iov, iovcnt);
			D("real_writev(%d, %p, %d) = %d\n", fd, iov, iovcnt, ret);

			if(ret == 0) {
				D("ret = %d, errno = %d\n", ret, errno);

				if(errno == EAGAIN)
					ITX_DELAYED_CONTINUE;
			}

			break;
		)
		
		ActCacheClear(ActCache_Write, fd);
        DT("leave: %d\n", ret);
		return ret;
	} else {
        size_t ret = real_writev(fd, iov, iovcnt);
        DT("leave: %d\n", ret);
		return ret;
	}
}

